"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var baseplatform_1 = require("./baseplatform");
var emu_1 = require("./emu");
var StateRecorderImpl = /** @class */ (function () {
    function StateRecorderImpl(platform) {
        this.checkpointInterval = 60;
        this.maxCheckpoints = 120;
        this.reset();
        this.platform = platform;
    }
    StateRecorderImpl.prototype.reset = function () {
        this.checkpoints = [];
        this.framerecs = [];
        this.frameCount = 0;
        this.lastSeekFrame = 0;
        if (this.callbackStateChanged)
            this.callbackStateChanged();
    };
    StateRecorderImpl.prototype.frameRequested = function () {
        var controls = {
            controls: this.platform.saveControlsState(),
            seed: emu_1.getNoiseSeed()
        };
        var requested = false;
        // are we replaying? then we don't need to save a frame, just replace controls
        if (this.lastSeekFrame < this.frameCount) {
            this.loadControls(this.lastSeekFrame);
        }
        else {
            // record the control state, if available
            if (this.platform.saveControlsState) {
                this.framerecs.push(controls);
            }
            // time to save next frame?
            requested = (this.frameCount++ % this.checkpointInterval) == 0;
        }
        this.lastSeekFrame++;
        if (this.callbackStateChanged)
            this.callbackStateChanged();
        return requested;
    };
    StateRecorderImpl.prototype.numFrames = function () {
        return this.frameCount;
    };
    StateRecorderImpl.prototype.currentFrame = function () {
        return this.lastSeekFrame;
    };
    StateRecorderImpl.prototype.recordFrame = function (state) {
        this.checkpoints.push(state);
        if (this.callbackNewCheckpoint)
            this.callbackNewCheckpoint(state);
        // checkpoints full?
        if (this.checkpoints.length > this.maxCheckpoints) {
            this.checkpoints.shift(); // remove 1st checkpoint
            this.framerecs = this.framerecs.slice(this.checkpointInterval);
            this.lastSeekFrame -= this.checkpointInterval;
            this.frameCount -= this.checkpointInterval;
            if (this.callbackStateChanged)
                this.callbackStateChanged();
        }
    };
    StateRecorderImpl.prototype.getStateAtOrBefore = function (frame) {
        var bufidx = Math.floor(frame / this.checkpointInterval);
        var foundidx = bufidx < this.checkpoints.length ? bufidx : this.checkpoints.length - 1;
        var foundframe = foundidx * this.checkpointInterval;
        return { frame: foundframe, state: this.checkpoints[foundidx] };
    };
    StateRecorderImpl.prototype.loadFrame = function (seekframe) {
        if (seekframe == this.lastSeekFrame)
            return seekframe; // already set to this frame
        // TODO: what if < 1?
        var _a = this.getStateAtOrBefore(seekframe - 1), frame = _a.frame, state = _a.state;
        if (state) {
            this.platform.pause();
            this.platform.loadState(state);
            while (frame < seekframe) {
                if (frame < this.framerecs.length) {
                    this.loadControls(frame);
                }
                frame++;
                this.platform.advance(frame < seekframe); // TODO: infinite loop?
            }
            this.lastSeekFrame = seekframe;
            return seekframe;
        }
        else {
            return 0;
        }
    };
    StateRecorderImpl.prototype.loadControls = function (frame) {
        if (this.platform.loadControlsState)
            this.platform.loadControlsState(this.framerecs[frame].controls);
        emu_1.setNoiseSeed(this.framerecs[frame].seed);
    };
    StateRecorderImpl.prototype.getLastCheckpoint = function () {
        return this.checkpoints.length && this.checkpoints[this.checkpoints.length - 1];
    };
    return StateRecorderImpl;
}());
exports.StateRecorderImpl = StateRecorderImpl;
// PROFILER
var PROFOP_READ = 0x100000;
var PROFOP_WRITE = 0x200000;
var PROFOP_INTERRUPT = 0x400000;
var EmuProfilerImpl = /** @class */ (function () {
    function EmuProfilerImpl(platform) {
        this.frame = null;
        this.output = { frame: null };
        this.i = 0;
        this.lastsl = 9999;
        this.starti = 0;
        this.platform = platform;
    }
    EmuProfilerImpl.prototype.start = function () {
        var _this = this;
        if (this.platform instanceof baseplatform_1.BasePlatform)
            this.platform.profiler = this;
        this.platform.setBreakpoint('profile', function () {
            var c = _this.platform.getCPUState();
            _this.log(c.EPC || c.PC);
            return false; // profile forever
        });
        this.output = { frame: null };
        return this.output;
    };
    EmuProfilerImpl.prototype.log = function (op) {
        var sl = this.platform.getRasterScanline();
        if (sl != this.lastsl) {
            if (this.frame) {
                this.frame.lines.push({ start: this.starti, end: this.i - 1 });
            }
            if (sl < this.lastsl) {
                this.output.frame = this.frame;
                this.frame = { iptab: new Uint32Array(0x8000), lines: [] }; // TODO: const
                this.i = 0;
            }
            this.starti = this.i;
            this.lastsl = sl;
        }
        this.frame.iptab[this.i++] = op;
    };
    EmuProfilerImpl.prototype.stop = function () {
        this.platform.clearBreakpoint('profile');
        if (this.platform instanceof baseplatform_1.BasePlatform)
            this.platform.profiler = null;
    };
    // TODO?
    EmuProfilerImpl.prototype.logRead = function (a) {
        this.log(a | PROFOP_READ);
    };
    EmuProfilerImpl.prototype.logWrite = function (a) {
        this.log(a | PROFOP_WRITE);
    };
    EmuProfilerImpl.prototype.logInterrupt = function (a) {
        this.log(a | PROFOP_INTERRUPT);
    };
    return EmuProfilerImpl;
}());
exports.EmuProfilerImpl = EmuProfilerImpl;
//# sourceMappingURL=recorder.js.map